@@ -7,14 +7,6 @@ package Apache2::Controller::Log::DetectAbortedConnection;
 Apache2::Controller::Log::DetectAbortedConnection -
 helper handler for detecting cancelled connections to the client.
 
-=head1 VERSION
-
-Version 0.110.000 - BETA TESTING (ALPHA?)
-
-=cut
-
-our $VERSION = version->new('0.110.000');
-
 =head1 DESCRIPTION
 
 You don't need to use this handler.
@@ -41,7 +33,7 @@ use Apache2::Const -compile => qw( OK );
 
 sub handler {
     my ($r) = @_;
-    $r->notes->{connection_closed} ||= $r->connection->aborted();
+    $r->notes->{connection_closed} = $r->connection->aborted();
     return Apache2::Const::OK;
 }
 
