@@ -4,14 +4,6 @@ package Apache2::Controller::Session::Cookie;
 
 Apache2::Controller::Session::Cookie - track a sessionid with a cookie in A2C
 
-=head1 VERSION
-
-Version 0.110.000 - BETA TESTING (ALPHA?)
-
-=cut
-
-our $VERSION = version->new('0.110.000');
-
 =head1 SYNOPSIS
 
 See L<Apache2::Controller::Session> for detailed setup example.
@@ -31,16 +23,25 @@ a cookie.
 
 =head1 DIRECTIVES
 
-=over 4 
+=over4 
 
-=item A2C_Session_Cookie_Opts
+=item A2CSessionCookieName
+
+=item A2CSessionCookieExpires
+
+See L<Apache2::Cookie> for valid fixed-duration strings.
+
+=item A2CSessionCookiePath
+
+Restrict the cookie path to something other than the default.
+
+=item A2CSessionCookieSecure
+
+Set the 'secure' flag on the cookie, which means it works only
+when transmitted by HTTPS.
 
 =back
 
-L<Apache2::Controller::Directives>
-
-L<Apache2::Cookie>
-
 =head1 METHODS
 
 These methods must by implemented by any 
@@ -49,7 +50,7 @@ L<Apache2::Controller::Session> subclass.
 =cut
 
 use strict;
-use warnings FATAL => 'all';
+use warnings;
 use English '-no_match_vars';
 
 use base qw( Apache2::Controller::Session );
@@ -77,9 +78,8 @@ Sets C<<$r->pnotes->{session_cookie}>> to be the Apache2::Cookie object.
 sub get_session_id {
     my ($self) = @_;
 
-    my %copts = %{ $self->get_directive('A2C_Session_Cookie_Opts') || { } }; 
-    $copts{name} ||= $DEFAULT_COOKIE_NAME;
-    my $cookie_name = $copts{name};
+    my $cookie_name = $self->get_directive('A2CSessionCookieName') 
+        || $DEFAULT_COOKIE_NAME;
     
     my $jar = $self->get_cookie_jar();
 
@@ -113,7 +113,11 @@ sub set_session_id {
 
     my $directives = $self->get_directives();
 
-    my %copts = %{ $self->get_directive('A2C_Session_Cookie_Opts') || { } }; 
+    my %copts 
+      = map {($_->[0] => $directives->{$_->[1]})}
+        grep exists $directives->{$_->[1]},
+        map [$_ => 'A2CSessionCookie'.ucfirst($_)],
+        qw( name expires secure domain path );
     $copts{name} ||= $DEFAULT_COOKIE_NAME;
 
     DEBUG(sub {"Creating session cookie with opts:\n".Dump(\%copts)});
